local RayUI, E, _, V, P, g = unpack(select(2, ...))
local RCH = RayUI:GetModule('RayUIChat')

--Cache global variables
--Lua functions
--WoW API / Variables
local _G, hooksecurefunc, select, pairs = _G, hooksecurefunc, select, pairs
local GetCursorPosition = GetCursorPosition
local InCombatLockdown = InCombatLockdown

-- GLOBALS:

RCH.autoHideTimer = 0

local NOT_FOUND_MATCH = ERR_CHAT_PLAYER_NOT_FOUND_S:format('(.+)')
local timeout = 0
local whentoshow={	
    "say", "emote", "text_emote", "yell",
    "monster_emote", "monster_party", "monster_say", "monster_whisper", "monster_yell",
    "party", "party_leader",
    "whisper", "system", "channel", "loot",
    "guild", "officer",
    "instance_chat", "instance_chat_leader",
    "raid", "raid_leader", "raid_warning",
    "bn_whisper",
    "bn_inline_toast_broadcast",
}

local channelNumbers = {
    [1] = true,
    [2] = true,
    [3] = true,
    [4] = true,
}

local function HideChatPanel()
	if not E.db[_G.LeftChatToggleButton.parent:GetName()..'Faded'] then
		_G.HideLeftChat()
	end
	if not E.db[_G.RightChatToggleButton.parent:GetName()..'Faded'] then
		_G.HideRightChat()
	end
end

local function ShowChatPanel()
	if E.db[_G.LeftChatToggleButton.parent:GetName()..'Faded'] then
		_G.HideLeftChat()
	end
	if E.db[_G.RightChatToggleButton.parent:GetName()..'Faded'] then
		_G.HideRightChat()
	end
end

local function CheckCursorPosition()
	local UIScale = _G.UIParent:GetScale()
	local x, y = GetCursorPosition()
	x = x/UIScale
	y = y/UIScale
	local IsOnLeftPanel = ( x > _G.LeftChatPanel:GetLeft() and x < _G.LeftChatPanel:GetLeft() + _G.LeftChatPanel:GetWidth() ) and ( y > _G.LeftChatPanel:GetBottom() and y < _G.LeftChatPanel:GetBottom() + _G.LeftChatPanel:GetHeight() )
	local IsOnRightPanel = ( x > _G.RightChatPanel:GetLeft() and x < _G.RightChatPanel:GetLeft() + _G.RightChatPanel:GetWidth() ) and ( y > _G.RightChatPanel:GetBottom() and y < _G.RightChatPanel:GetBottom() + _G.RightChatPanel:GetHeight() )
	return IsOnLeftPanel or IsOnRightPanel
end

function RCH:OnEvent(event, msg, ...)
    if event == "CHAT_MSG_CHANNEL" and (msg:find("^%^LFW_") or select(8,...):find("友团")) then return end
	if event == "CHAT_MSG_SYSTEM" and msg:match(NOT_FOUND_MATCH) then return end
    timeout = 0
    ShowChatPanel()
end

function RCH:OnUpdate(elapsed)
	if CheckCursorPosition() then
		timeout = 0
		ShowChatPanel()
		return
	end
	if not InCombatLockdown() then
		timeout = timeout + elapsed
		if timeout > E.db.rayui.chat.autoHideTimeout then
			HideChatPanel()
		end
	end
end

function RCH:AutoHide()
	ShowChatPanel()
	self:Configure_AutoHide()
end

function RCH:ChatEdit_OnEnterPressed()
	timeout = 0
    ShowChatPanel()
end

function RCH:Configure_AutoHide()
	if E.db.rayui.chat.autoHide then
		for _, event in pairs(whentoshow) do
			if(not event:match("[A-Z]")) then
				event = "CHAT_MSG_"..event:upper()
			end
			self:RegisterEvent(event, "OnEvent")
		end
		self:RegisterEvent("PLAYER_REGEN_DISABLED", "OnEvent")
		self:RegisterEvent("BN_FRIEND_ACCOUNT_ONLINE", "OnEvent")
		self:RegisterEvent("BN_FRIEND_ACCOUNT_OFFLINE", "OnEvent")
		self:RegisterEvent("BN_CUSTOM_MESSAGE_CHANGED", "OnEvent")
		self:RegisterEvent("BN_FRIEND_INVITE_ADDED", "OnEvent")
		self:RegisterEvent("BN_FRIEND_INVITE_LIST_INITIALIZED", "OnEvent")
		self.autoHideTimer = self:ScheduleRepeatingTimer("OnUpdate", 0.5, 0.5)
		self:SecureHook('ChatEdit_OnEnterPressed')
	else
		timeout = 0
		ShowChatPanel()
		for _, event in pairs(whentoshow) do
			if(not event:match("[A-Z]")) then
				event = "CHAT_MSG_"..event:upper()
			end
			self:UnregisterEvent(event)
		end
		self:UnregisterEvent("PLAYER_REGEN_DISABLED")
		self:UnregisterEvent("BN_FRIEND_ACCOUNT_ONLINE")
		self:UnregisterEvent("BN_FRIEND_ACCOUNT_OFFLINE")
		self:UnregisterEvent("BN_CUSTOM_MESSAGE_CHANGED")
		self:UnregisterEvent("BN_FRIEND_INVITE_ADDED")
		self:UnregisterEvent("BN_FRIEND_INVITE_LIST_INITIALIZED")
		self:CancelTimer(self.autoHideTimer)
		self:Unhook('ChatEdit_OnEnterPressed')
	end
end
